use std::error::Error as StdError;
use std::io::Error as IoError;
use std::net::AddrParseError;
use hyper::Error as HyperError;

use {RawResponse, StatusCode};


error_chain! {
    types {
        SusanooError, SusanooErrorKind, ResultExt, SusanooResult;
    }

    foreign_links {
        AddrParse(AddrParseError);
        Hyper(HyperError);
        StdIo(IoError);
    }
}


/// Error type thrown during handling
pub enum HandleError {
    Status(StatusCode),
    Internal(Box<StdError + Send + 'static>),
    Raw(RawResponse),
}

impl HandleError {
    pub fn not_found() -> Self {
        HandleError::Status(StatusCode::NotFound)
    }

    pub fn bad_request() -> Self {
        HandleError::Status(StatusCode::BadRequest)
    }

    pub fn internal<E: StdError + Send + 'static>(err: E) -> Self {
        HandleError::Internal(Box::new(err))
    }
}
